#!/usr/bin/env ruby
#
# amaroK Script for fetching song lyrics from http://music.naver.com.
# Ported from amaroK's contextbrowser.cpp.
#
# (c) 2007 Jaehyun Yeom <only2sea@dreamwiz.com>
#
# License: GNU General Public License V2

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))
require 'amarok_naver'
require 'rexml/document'

@app_name = "Lyrics_Naver"

def showLyrics( lyrics )
  # Important, otherwise we might execute arbitrary nonsense in the DCOP call
  lyrics.gsub!( '"', "'" )
  lyrics.gsub!( '`', "'" )
  
  `dcop amarok contextbrowser showLyrics "#{lyrics}"`
end

def notFound()
  doc = REXML::Document.new( "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" )
  root = doc.add_element( "suggestions" )
  root.add_attribute( "page_url", @page_url )
  xml = ""
  doc.write( xml )
  showLyrics( xml )
end

def exportXml(artist, title, lyrics)
  doc = REXML::Document.new( "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" )
  root = doc.add_element("lyrics")

  root.add_attribute("page_url", @page_url)
  root.add_attribute("artist", artist)
  root.add_attribute("title", title)
  root.text = lyrics
  xml = ""
  doc.write(xml)
  showLyrics(xml)
end

def fetchLyrics(artist, title, url)
  query = artist + ' ' + title
  fetArtist, fetTitle, fetLyric = (url == "" ? AmarokNaver.getLyric(query) : AmarokNaver.getLyricFromUrl(url))
  if fetArtist == nil or fetTitle == nil or fetLyric == nil then
    notFound
  else
    exportXml(fetArtist, fetTitle, fetLyric)
  end
end

loop do
  message = gets().chomp()
  command = /[A-Za-z]*/.match( message ).to_s()

  case command
  when "configure"
    msg  = 'This script does not require any configuration.'
    `dcop amarok playlist popupMessage "#{msg}"`

  when "fetchLyrics"
    args = message.split()
    
    artist = args[1]
    title  = args[2]
    
    fetchLyrics( artist, title, "" )

  when "fetchLyricsByUrl"
    url = message.split()[1]

    fetchLyrics( "", "", url )
  end
end

